/*****************************************************************************
 *   The proposed solution for LPCXpresso Experiment Kit, Lab 12e
 *
 *   Copyright(C) 2013, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/

#include "LPC11xx.h"
#include "type.h"
#include "board.h"
#include "gpio.h"
#include "delay.h"
#include "adc.h"
#include "pwm.h"
#include "spi.h"

#define SEG_A   0x01
#define SEG_B   0x02
#define SEG_C   0x04
#define SEG_D   0x08
#define SEG_E   0x10
#define SEG_F   0x20
#define SEG_G   0x40

const uint8_t segments[10] = {
    SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F        ,  //0 = A,B,C,D,E,F
            SEG_B | SEG_C                                ,  //1 = B,C
    SEG_A | SEG_B |         SEG_D | SEG_E |         SEG_G,  //2 = A,B,D,E,G
    SEG_A | SEG_B | SEG_C | SEG_D |                 SEG_G,  //3 = A,B,C,D,G
            SEG_B | SEG_C |                 SEG_F | SEG_G,  //4 = B,C,F,G
    SEG_A |         SEG_C | SEG_D |         SEG_F | SEG_G,  //5 = A,C,D,F,G
    SEG_A |         SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,  //6 = A,C,D,E,F,G
    SEG_A | SEG_B | SEG_C                                ,  //7 = A,B,C
    SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,  //8 = A,B,C,D,E,F,G
    SEG_A | SEG_B | SEG_C | SEG_D |         SEG_F | SEG_G}; //9 = A,B,C,D,F,G

/* Declare variable to store digit outputs */
volatile uint8_t digitSegments[2];

uint8_t convert00_99(uint16_t inputVal)
{
  return (inputVal * 100) / 1024;
}

/* Define Interrupt Service Routine for 32-bit timer #1 */
void TIMER32_1_IRQHandler(void)    //name of function is predefined
{
  //counter that indicate active digit (numbered 0 and 1)
  static uint8_t activeDigit;

  if (activeDigit == 0)
  {
    //Disconnect anode of digit #0 (pull control signal high)
    GPIOSetValue( DIG1_PORT, DIG1_PIN, 1);

    //Send segment outputs (via SPI) for digit 1
    GPIOSetValue( SSEL_GPIO_4_PORT, SSEL_GPIO_4_PIN, SSEL_LOW);
    SSP0Send( &digitSegments[0], 1 );
    GPIOSetValue( SSEL_GPIO_4_PORT, SSEL_GPIO_4_PIN, SSEL_HIGH);

    //Connect anode of digit #1 (pull control signal low)
    GPIOSetValue( DIG2_PORT, DIG2_PIN, 0);

    activeDigit = 1;
  }

  else
  {
    //Disconnect anode of digit #1 (pull control signal high)
    GPIOSetValue( DIG2_PORT, DIG2_PIN, 1);

    //Send segment outputs (via SPI) for digit 0
    GPIOSetValue( SSEL_GPIO_4_PORT, SSEL_GPIO_4_PIN, SSEL_LOW);
    SSP0Send( &digitSegments[1], 1 );
    GPIOSetValue( SSEL_GPIO_4_PORT, SSEL_GPIO_4_PIN, SSEL_HIGH);

    //Connect anode of digit #0 (pull control signal low)
    GPIOSetValue( DIG1_PORT, DIG1_PIN, 0);

    activeDigit = 0;
  }

  //clear timer interrupt
  LPC_TMR32B1->IR = 0x01;   //write with bit 0 set to clear interrupt from MR0-register
}

int main (void)
{
  uint32_t analogValue;

  //setup pins for controlling 7-segment display
  GPIOSetDir( DIG1_PORT, DIG1_PIN, GPIO_OUTPUT);
  GPIOSetValue( DIG1_PORT, DIG1_PIN, 1);
  GPIOSetDir( DIG2_PORT, DIG2_PIN, GPIO_OUTPUT);
  GPIOSetValue( DIG2_PORT, DIG2_PIN, 1);

  //Set SW2/SW3 pins as inputs
  GPIOSetDir( SW2_PORT, SW2_PIN, GPIO_INPUT);
  GPIOSetDir( SW3_PORT, SW3_PIN, GPIO_INPUT);

  //Extra, turn buzzer pin to input
  GPIOSetDir( BUZZ_PORT, BUZZ_PIN, GPIO_OUTPUT);
  GPIOSetValue( BUZZ_PORT, BUZZ_PIN, BUZZ_OFF);

  SSP0Init(1500000);

  //Handle SSEL signal - note: same as LED1, so technically not needed to do this
  GPIOSetDir( SSEL_GPIO_4_PORT, SSEL_GPIO_4_PIN, GPIO_OUTPUT);
  GPIOSetValue( SSEL_GPIO_4_PORT, SSEL_GPIO_4_PIN, 1);

  //Initialize ADC peripheral and pin-mixing
  ADCInit(4500000);  //4.5MHz ADC clock

  //get initial value
  analogValue = getADC(AIN0);   //AIN0 = trimming pot for intensity

  /* setup 32-bit timer #1 to generate continous interrupts every 1 ms = 1000 Hz */
  LPC_SYSCON->SYSAHBCLKCTRL |= (1<<10); /* Enable 32-bit timer #1 clock */
  LPC_TMR32B1->TCR = 0x02;             /* reset timer */
  LPC_TMR32B1->PR  = 0x00;             /* set prescaler to zero */

  //(SystemCoreClock/LPC_SYSCON->SYSAHBCLKDIV) = 48000000 => Timer clock is 48MHz
  LPC_TMR32B1->MR0 = 1 * ((SystemCoreClock/LPC_SYSCON->SYSAHBCLKDIV)/ 1000);

  LPC_TMR32B1->IR  = 0xff;             /* reset all interrupts (not needed) */
  LPC_TMR32B1->MCR = (0x01) | (0x02);  /* generate irq and reset counter on match */
  LPC_TMR32B1->TCR = 0x01;             /* start timer */

  digitSegments[0] = 0xFF;
  digitSegments[1] = 0xFF;

  /* enable 16-bit timer #0 interrupt */
  NVIC_EnableIRQ(TIMER_32_1_IRQn);

  /* enter forever
   *  loop – let interrupt handle processing */
  while(1)
  {
    uint8_t outputValue;

    outputValue = convert00_99(getADC(AIN0));
    digitSegments[0] = ~segments[outputValue % 10];
    digitSegments[1] = ~segments[outputValue / 10];

    delayMS(100);
  }

  return 0;
}
