/*****************************************************************************
 *   Placeholder for timer callback related functions.
 *   Part of the proposed solutions for LPCXpresso Experiment Kit
 *
 *   Copyright(C) 2013, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/

#include "LPC11xx.h"
#include "type.h"
#include "timerCB.h"

/* Declare function pointer (to a void-void function) for the callback function */
static volatile void (*pCB)(void) = NULL;
static volatile uint8_t repeatCB;

/* Define Interrupt Service Routine for 32-bit timer #1 */
void TIMER32_1_IRQHandler (void)    //name of function is predefined
{
  /* check if function pointer is value (not equal to NULL) */
  if(pCB != NULL)
  {
    /* call function pointer = call callback function */
    pCB();   //also valid syntax: (*pCB)();

  }

  //check it repeated callback, if not, stop counter
  if (!repeatCB)
  {
    LPC_TMR32B1->TCR = 0;
    /* invalidate function pointer */
    pCB = NULL;
  }

  //clear timer interrupt
  LPC_TMR32B1->IR = 0x01;   //write with bit 0 set to clear interrupt from MR0-register
}

/*****************************************************************************
** Function name:    registerCbAndDelay
** Descriptions:     This function setup 32-bit timer #1 to generate
**                   an interrupt after specified time and then call a
**                   registered callback function.
** parameters:       delay in ms and callback function pointer
** Returned value:   None
**
*****************************************************************************/
void registerCbAndDelay( uint16_t delayInMS, uint8_t repeat, void (*pF)(void))
{
  /* register callback function */
  pCB      = pF;
  repeatCB = repeat;

  /* setup timer to fire in ‘delayInMS’ ms */
  LPC_SYSCON->SYSAHBCLKCTRL |= (1<<10); /* Enable 32-bit timer #1 clock */
  LPC_TMR32B1->TCR = 0x02;             /* reset timer */
  LPC_TMR32B1->PR  = 0x00;             /* set prescaler to zero */

  //(SystemCoreClock/LPC_SYSCON->SYSAHBCLKDIV) = 48000000 => Timer clock is 48MHz
  LPC_TMR32B1->MR0 = delayInMS * ((SystemCoreClock/LPC_SYSCON->SYSAHBCLKDIV)/ 1000);

  LPC_TMR32B1->IR  = 0xff;             /* reset all interrupts (not needed) */
  LPC_TMR32B1->MCR = (0x01) | (0x02);  /* generate irq and reset counter on match */
  LPC_TMR32B1->TCR = 0x01;             /* start timer */

  /* enable 32-bit timer #1 interrupt */
  NVIC_EnableIRQ(TIMER_32_1_IRQn);
}

/*****************************************************************************
** Function name:    checkIfCbActive
** Descriptions:     This function checks if the callback is active
** parameters:       None
** Returned value:   TRUE if callback is active, else FLASE
**
*****************************************************************************/
uint8_t checkIfCbActive( void )
{
  if (pCB == NULL)
    return FALSE;
  else
    return TRUE;
}

/*****************************************************************************
** Function name:    stopCb
** Descriptions:     This function stops a repeat-callback
** parameters:       None
** Returned value:   None
**
*****************************************************************************/
void stopCb( void )
{
  //stop timer
  LPC_TMR32B1->TCR = 0;

  /* disable 32-bit timer #1 interrupt */
  NVIC_DisableIRQ(TIMER_32_1_IRQn);

  //invalidate function pointer
  pCB = NULL;
}
