/*****************************************************************************
 *   The proposed solution for LPCXpresso Experiment Kit, Lab 15a
 *
 *   Copyright(C) 2013, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/

#include "stdio.h"
#include "LPC11xx.h"
#include "type.h"
#include "board.h"
#include "gpio.h"
#include "delay.h"
#include "adc.h"
#include "pwm.h"
#include "spi.h"
#include "timerCB.h"
#include "i2c.h"
#include "pca9532.h"
#include "uart.h"
#include "xbee.h"

/*
 * Application configuration
 *
 * CFG_ACT_AS_COORDINATOR - (1) - Configure the XBee module to act as a coordinator.
 *                          (0) - Configure the XBee module to act as an end-device
 */
#define CFG_ACT_AS_COORDINATOR (0)

/*
 * RF message IDs, add your own here but make sure that the coordinator
 * and the nodes have the same numbering.
 */
#define RFPT_SET_LED (1)


// Forward declarations
static void xbeeUp(uint8_t up);
static void xbeeNode(uint32_t addrHi, uint32_t addrLo, uint8_t rssi);
static void xbeeTxStatus(uint8_t frameId, xbeeTxStatus_t error);
static void xbeeData(uint32_t addrHi, uint32_t addrLo, uint8_t rssi,
    uint8_t* buf, uint8_t len);


static xbee_callb_t callbacks = {
    xbeeUp,
    xbeeNode,
    xbeeTxStatus,
    xbeeData
};

static uint8_t devIsReady = 0;

volatile uint32_t ms_ticks = 0;


/*****************************************************************************
 ** Function name:  SysTick_Handler
 **
 ** Descriptions:   Interrupt handler. Updates the ms_ticks variable to hold
 **                 the number of milliseconds since start. This will be
 **                 reasonably accurate and is used by the Xbee driver to
 **                 handle timeouts.
 **
 ** parameters:     None
 ** Returned value: The time in milliseconds
 **
 *****************************************************************************/
void SysTick_Handler(void) {
    ms_ticks += 10;
}

/*****************************************************************************
 ** Function name:  xbeeUp
 **
 ** Descriptions:   XBee node up/down callback.
 **
 ** parameters:     up will be 1 if the node is up, 0 if it is down
 ** Returned value: None
 **
 *****************************************************************************/
static void xbeeUp(uint8_t up)
{
  printf("RF: Xbee Up (%d)\r\n", up);
  devIsReady = up;
}

/*****************************************************************************
 ** Function name:  xbeeNode
 **
 ** Descriptions:   XBee node discover callback. Will be called as a response
 **                 to a Xbee node discovery request. All found nodes are
 **                 reported back one-by-one through this callback.
 **
 ** parameters:     addrHi - upper 32 bits of the 64-bit node address,
 **                 addrLo - lower 32 bits of the 64-bit node address,
 **                 rssi   - signal strength
 ** Returned value: None
 **
 *****************************************************************************/
static void xbeeNode(uint32_t addrHi, uint32_t addrLo, uint8_t rssi)
{
  printf("RF: Node %x:%x rssi=%d\r\n", addrHi, addrLo, rssi);
}

/*****************************************************************************
 ** Function name:  xbeeTxStatus
 **
 ** Descriptions:   Transmit status callback. Called as a result of a packet
 **                 being sent from the Xbee node.
 **
 ** parameters:     frameId - ID of the frame that was sent,
 **                 status - status of the transmit request
 ** Returned value: None
 **
 *****************************************************************************/
static void xbeeTxStatus(uint8_t frameId, xbeeTxStatus_t status)
{
  if (status != XBEE_TX_STAT_OK) {
    printf("RF: [%d] TX failed %d\r\n", frameId, status);
  }
}

/*****************************************************************************
 ** Function name:  xbeeTxStatus
 **
 ** Descriptions:   Received data callback. Called when data has been received
 **                 by the Xbee node.
 **
 ** parameters:     addrHi - upper 32 bits of the 64-bit node address,
 **                 addrLo - lower 32 bits of the 64-bit node address,
 **                 rssi   - signal strength,
 **                 buf    - buffer containing the data,
 **                 len    - number of received bytes,
 ** Returned value: None
 **
 *****************************************************************************/
static void xbeeData(uint32_t addrHi, uint32_t addrLo, uint8_t rssi,
    uint8_t* buf, uint8_t len)
{
  int i = 0;
  printf("xbeeData %x:%x, rssi=%d, len=%d\r\n", addrHi, addrLo, rssi, len);

  if (len < 1) {
    return;
  }

  switch (buf[0]) {

  // Set LED request. This is a two byte request where the data
  // indicates if the LED should be turned on or off.
  case RFPT_SET_LED:
    if (len > 1) {
      if (buf[1] == 1) {
    	 GPIOSetValue(LED1_PORT, LED1_PIN, LED_ON);
      } else {
    	 GPIOSetValue(LED1_PORT, LED1_PIN, LED_OFF);
      }
    }
    break;

  default:
    for (i = 0; i < len; i++) {
      if (i > 0 && (i%8) == 0) {
        printf("\r\n");
      }
      printf("%x ", buf[i]);
    }
    printf("\r\n");

    break;
  }
}

/*****************************************************************************
 ** Function name:  sendSetLedRequest
 **
 ** Descriptions:   Broadcasts a request over Xbee to set the status of the LED
 **
 ** parameters:     ledOn - should the LED be lit or not
 ** Returned value: ERR_OK or an error code
 **
 *****************************************************************************/
static error_t sendSetLedRequest(uint8_t ledOn)
{
  uint8_t data[2];
  uint8_t id = 0;

  data[0] = RFPT_SET_LED;
  data[1] = ledOn;

  return xbee_send(XBEE_ADDRHI_BROADCAST, XBEE_ADDRLO_BROADCAST, data, 2, &id);
}

int main (void)
{
  error_t err;
  uint8_t state = 0;
  uint8_t oldState = !SW_PRESSED;

  //Set LED1-LED8 pins as outputs
  GPIOSetDir( LED1_PORT, LED1_PIN, GPIO_OUTPUT);
  GPIOSetValue( LED1_PORT, LED1_PIN, LED_OFF);
  GPIOSetDir( LED2_PORT, LED2_PIN, GPIO_OUTPUT);
  GPIOSetValue( LED2_PORT, LED2_PIN, LED_OFF);
  GPIOSetDir( LED3_PORT, LED3_PIN, GPIO_OUTPUT);
  GPIOSetValue( LED3_PORT, LED3_PIN, LED_OFF);
  GPIOSetDir( LED4_PORT, LED4_PIN, GPIO_OUTPUT);
  GPIOSetValue( LED4_PORT, LED4_PIN, LED_OFF);
  GPIOSetDir( LED5_PORT, LED5_PIN, GPIO_OUTPUT);
  GPIOSetValue( LED5_PORT, LED5_PIN, LED_OFF);
  GPIOSetDir( LED6_PORT, LED6_PIN, GPIO_OUTPUT);
  GPIOSetValue( LED6_PORT, LED6_PIN, LED_OFF);
  GPIOSetDir( LED7_PORT, LED7_PIN, GPIO_OUTPUT);
  GPIOSetValue( LED7_PORT, LED7_PIN, LED_OFF);
  GPIOSetDir( LED8_PORT, LED8_PIN, GPIO_OUTPUT);
  GPIOSetValue( LED8_PORT, LED8_PIN, LED_OFF);

  //Set SW2/SW3 pins as inputs
  GPIOSetDir( SW2_PORT, SW2_PIN, GPIO_INPUT);
  GPIOSetDir( SW3_PORT, SW3_PIN, GPIO_INPUT);

  //Extra, turn buzzer pin to input
  GPIOSetDir( BUZZ_PORT, BUZZ_PIN, GPIO_OUTPUT);
  GPIOSetValue( BUZZ_PORT, BUZZ_PIN, BUZZ_OFF);

  //Use systick to get an interrupt every 10ms
  SysTick_Config(SystemCoreClock / 100);

#if (CFG_ACT_AS_COORDINATOR == 1)
  printf("XBee demo - COORDINATOR\r\n");
  err = xbee_init(XBEE_COORDINATOR, &callbacks);
#else
  printf("XBee demo - NODE\r\n");
  err = xbee_init(XBEE_END_DEVICE, &callbacks);
#endif

  if (err != ERR_OK) {
	printf("Failed to initialize Xbee. Error code %d. Aborting...\n", err);
	while (1) {
		// wait forever
	}
  }

  while (1) {
    xbee_task();

    if (devIsReady != 0) {
      // check button state
      state = GPIOGetValue(SW2_PORT, SW2_PIN);
      if (oldState != state) {
        oldState = state;
        printf("Button: %u\r\n", state);

        sendSetLedRequest(state);
      }
    }
  }

  return 0;
}
