/*****************************************************************************
 *   Placeholder for UART related functions.
 *   Part of the proposed solutions for LPCXpresso Experiment Kit
 *
 *   Copyright(C) 2013, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/

#include "LPC11xx.h"
#include "type.h"
#include "uart.h"

static volatile uint8_t  txBuf[TX_BUFFER_SIZE];
static volatile uint32_t txHead = 0;
static volatile uint32_t txTail = 0;
static volatile uint8_t  txRunning = FALSE;

static volatile uint8_t  rxBuf[RX_BUFFER_SIZE];
static volatile uint32_t rxHead = 0;
static volatile uint32_t rxTail = 0;

/*****************************************************************************
** Function name:  UARTInit
**
** Descriptions:   Initialize UART0 port, setup pin select,
**                 clock, parity, stop bits, FIFO, etc.
**
** parameters:     UART baudrate
** Returned value: None
**
*****************************************************************************/
void UARTInit(uint32_t baudrate)
{
  uint32_t Fdiv;
  uint32_t regVal;

  NVIC_DisableIRQ(UART_IRQn);

  LPC_IOCON->PIO1_6 &= ~0x07;        /*  UART I/O config */
  LPC_IOCON->PIO1_6 |= 0x01;         /* UART RXD */
  LPC_IOCON->PIO1_7 &= ~0x07;
  LPC_IOCON->PIO1_7 |= 0x01;         /* UART TXD */

  /* Enable UART clock */
  LPC_SYSCON->SYSAHBCLKCTRL |= (1<<12);
  LPC_SYSCON->UARTCLKDIV = 0x1;      /* divided by 1 */

  LPC_UART->LCR = 0x83;              /* 8 bits, no Parity, 1 Stop bit */
  regVal = LPC_SYSCON->UARTCLKDIV;
  Fdiv = (((SystemCoreClock/LPC_SYSCON->SYSAHBCLKDIV)/regVal)/16)/baudrate;

  LPC_UART->DLM = Fdiv / 256;
  LPC_UART->DLL = Fdiv % 256;
  LPC_UART->LCR = 0x03;   /* DLAB = 0 */
  LPC_UART->FCR = 0x07;   /* Enable and reset TX and RX FIFO. */

  /* Read to clear the line status. */
  regVal = LPC_UART->LSR;

  /* Ensure a clean start, no data in either TX or RX FIFO. */
  while ( (LPC_UART->LSR & (LSR_THRE|LSR_TEMT)) != (LSR_THRE|LSR_TEMT) );
  while ( LPC_UART->LSR & LSR_RDR )
  {
    regVal = LPC_UART->RBR; /* Dump data from RX FIFO */
  }

  //initialize the transmit data queue
  txHead    = 0;
  txTail    = 0;
  txRunning = FALSE;

  //initialize the receive data queue
  rxHead    = 0;
  rxTail    = 0;

  /* Enable the UART Interrupt */
  NVIC_EnableIRQ(UART_IRQn);

  LPC_UART->IER = IER_RBR | IER_THRE | IER_RLS; /* Enable UART interrupt */
}

/*****************************************************************************
** Function name:   UART_IRQHandler
**
** Descriptions:    UART interrupt handler
**
** parameters:      None
** Returned value:    None
**
*****************************************************************************/
void UART_IRQHandler(void)
{
  volatile uint8_t IIRValue, LSRValue, statusReg;
  uint8_t Dummy = Dummy;
  volatile uint32_t tmpHead;
  volatile uint32_t tmpTail;

  statusReg = IIRValue = LPC_UART->IIR;

  IIRValue >>= 1;     /* skip pending bit in IIR */
  IIRValue &= 0x07;     /* check bit 1~3, interrupt identification */
  if (IIRValue == IIR_RLS)    /* Receive Line Status */
  {
    LSRValue = LPC_UART->LSR;
    /* Receive Line Status */
    if (LSRValue & (LSR_OE | LSR_PE | LSR_FE | LSR_RXFE | LSR_BI))
    {
      /* There are errors or break interrupt */
      /* Read LSR will clear the interrupt */
//      UARTStatus = LSRValue;
      Dummy = LPC_UART->RBR;  //Dummy read on RX to clear interrupt, then bail out
      return;
    }
    if (LSRValue & LSR_RDR) /* Receive Data Ready */
    {
      /* If no error on RLS, normal ready, save into the data buffer. */
      /* Note: read RBR will clear the interrupt */
      tmpHead = (rxHead + 1) & RX_BUFFER_MASK;
      rxHead  = tmpHead;

      if(tmpHead == rxTail)
        tmpHead = LPC_UART->RBR;         //dummy read to reset IRQ flag
      else
        rxBuf[tmpHead] = LPC_UART->RBR;  //will reset IRQ flag
    }
  }
  else if (IIRValue == IIR_RDA) /* Receive Data Available */
  {
    /* Receive Data Available */
    tmpHead = (rxHead + 1) & RX_BUFFER_MASK;
    rxHead  = tmpHead;

    if(tmpHead == rxTail)
      tmpHead = LPC_UART->RBR;         //dummy read to reset IRQ flag
    else
      rxBuf[tmpHead] = LPC_UART->RBR;  //will reset IRQ flag
  }
  else if (IIRValue == IIR_CTI) /* Character timeout indicator */
  {
    /* Character Time-out indicator */
//    UARTStatus |= 0x100;    /* Bit 9 as the CTI error */
  }
  else if (IIRValue == IIR_THRE)  /* THRE, transmit holding register empty */
  {
    //check if all data is transmitted
    if (txHead != txTail)
    {
      uint32_t bytesToSend;

      if (statusReg & 0xc0)
        bytesToSend = 16;    //FIFO enabled
      else
        bytesToSend = 1;     //no FIFO enabled

      do
      {
        //calculate buffer index
        tmpTail = (txTail + 1) & TX_BUFFER_MASK;

        txTail = tmpTail;
        LPC_UART->THR = txBuf[tmpTail];
      } while((txHead != txTail) && --bytesToSend);
    }

    //all data has been transmitted
    else
    {
      txRunning = FALSE;
      LPC_UART->IER &= ~IER_THRE;     //disable TX IRQ
    }
  }
}

/*****************************************************************************
 ** Function name:  UARTSendChar
 **
 ** Descriptions:   Send a byte/char of data to the UART 0 port
 **
 ** parameters:     byte to send
 ** Returned value: None
 **
 *****************************************************************************/
void UARTSendChar(uint8_t toSend)
{
  uint32_t tmpHead;

  //calculate head index
  tmpHead = (txHead + 1) & TX_BUFFER_MASK;

  //wait for free space in buffer
  while(tmpHead == txTail)
    ;

  //disable TX IRQ
  LPC_UART->IER &= ~IER_THRE;

  if(txRunning == TRUE)
  {
    txBuf[tmpHead] = toSend;
    txHead         = tmpHead;
  }
  else
  {
    txRunning = TRUE;

    /* Extra check - should not be needed: THRE status, contain valid data */
    while ( !(LPC_UART->LSR & LSR_THRE) )
      ;
    LPC_UART->THR = toSend;
  }

  //enable TX IRQ
  LPC_UART->IER |= IER_THRE;
}

/*****************************************************************************
 ** Function name:  UARTSendString
 **
 ** Descriptions:   Send a null-terminated string to UART 0 port
 **
 ** parameters:     byte to send
 ** Returned value: None
 **
 *****************************************************************************/
void UARTSendString(uint8_t *pStr)
{
  while (*pStr != 0)
    UARTSendChar(*pStr++);

}

/*****************************************************************************
 ** Function name:  UARTSendBuffer
 **
 ** Descriptions:   Send a number of bytes/chars of data to UART 0 port
 **
 ** parameters:     byte to send
 ** Returned value: None
 **
 *****************************************************************************/
void UARTSendBuffer(uint8_t *pBuf, uint16_t length)
{
  while (length-- > 0)
    UARTSendChar(*pBuf++);
}

/*****************************************************************************
 ** Function name:  UARTGetCharBlock
 **
 ** Descriptions:   Receive a char from UART 0
 **
 ** parameters:     None
 ** Returned value: Received char
 **
 *****************************************************************************/
uint8_t UARTGetCharBlock(void)
{
  uint8_t rxChar;

  //wait for a character to be available
  while(UARTGetChar(&rxChar) == FALSE)
    ;
  return rxChar;
}

/*****************************************************************************
 ** Function name:  UARTGetChar
 **
 ** Descriptions:   Receive a char from UART 0
 **
 ** parameters:     pointer to where to store received char
 ** Returned value: TRUE if char received, else FALSE
 **
 *****************************************************************************/
uint8_t UARTGetChar(uint8_t *pRxChar)
{
  uint32_t tmpTail;

  /* check if buffer is empty */
  if(rxHead == rxTail)
    return FALSE;

  tmpTail = (rxTail + 1) & RX_BUFFER_MASK;
  rxTail  = tmpTail;

  *pRxChar = rxBuf[tmpTail];
  return TRUE;
}
